﻿using System;
using System.Globalization;
using System.Net;
using Subkismet.Waegis.Client.Components;
using Subkismet.Components;
using Subkismet.Resources;

namespace Subkismet.Waegis.Client
{
    /// <summary>
    /// Used to manage the working process with Waegis API and its methods
    /// </summary>
    [Serializable]
    public class WaegisClient
    {
        #region Fields

        [NonSerialized]
        private HttpClient _httpClient = null;

        static readonly string _version = typeof(HttpClient).Assembly.GetName().Version.ToString();
        const string initializeUrlFormat = "http://api.waegis.com/web/1.0/{0}/initialize";
        const string checkUrlFormat = "http://api.waegis.com/web/1.0/{0}/check";
        const string reportSpamUrlFormat = "http://api.waegis.com/web/1.0/{0}/report-spam";
        const string reportHamUrlFormat = "http://api.waegis.com/web/1.0/{0}/report-ham";

        Uri _initializeUrl = null;
        Uri _checkUrl = null;
        Uri _reportSpamUrl = null;
        Uri _reportHamUrl = null;

        #endregion

        #region Properties

        string _apiKey;
        /// <summary>
        /// The API key
        /// </summary>
        string ApiKey
        {
            get
            {
                return this._apiKey ?? string.Empty;
            }
            set
            {
                this._apiKey = value ?? string.Empty;
                SetServiceUrls();
            }
        }

        Uri _siteUrl;
        /// <summary>
        /// The site URL
        /// </summary>
        Uri SiteUrl
        {
            get
            {
                return this._siteUrl;
            }
            set
            {
                this._siteUrl = value;
            }
        }

        IWebProxy _proxy;
        /// <summary>
        /// The proxy object to be used for the requests
        /// </summary>
        IWebProxy Proxy
        {
            get
            {
                return this._proxy;
            }
            set
            {
                this._proxy = value;
            }
        }

        int _timeout = 20000;
        /// <summary>
        /// Timeout in miliseconds
        /// </summary>
        int Timeout
        {
            get
            {
                return this._timeout;
            }
            set
            {
                this._timeout = value;
            }
        }

        string _userAgent;
        /// <summary>
        /// The user agent
        /// </summary>
        string UserAgent
        {
            get
            {
                return this._userAgent;
            }
            set
            {
                this._userAgent = value;
            }
        }

        #endregion

        #region Public Constrcutors

        /// <summary>
        /// Public constructor
        /// </summary>
        /// <param name="apiKey">API key</param>
        /// <param name="siteUrl">Site URL</param>
        /// <param name="httpClient">HTTP client to use for requests</param>
        /// <param name="applicationName">The name of the application</param>
        /// <param name="applicationVersion">The version of the application</param>
        public WaegisClient(string apiKey, Uri siteUrl, HttpClient httpClient, string applicationName, string applicationVersion)
        {
            if (string.IsNullOrEmpty(apiKey))
                throw new ArgumentNullException("siteKey", Messages.ArgumentNull_String);

            if (siteUrl == null)
                throw new ArgumentNullException("siteUrl", Messages.ArgumentNull_Uri);

            if (httpClient == null)
                throw new ArgumentNullException("httpClient", Messages.ArgumentNull_Generic);

            this._apiKey = apiKey;
            this._siteUrl = siteUrl;
            this._httpClient = httpClient;
            this._userAgent = BuildUserAgent(applicationName, applicationVersion);

            SetServiceUrls();
        }

        /// <summary>
        /// Public constructor
        /// </summary>
        /// <param name="apiKey">API key</param>
        /// <param name="siteUrl">Site URL</param>
        /// <param name="applicationName">The name of the application</param>
        /// <param name="applicationVersion">The version of the application</param>
        public WaegisClient(string apiKey, Uri siteUrl, string applicationName, string applicationVersion)
            : this(apiKey, siteUrl, new HttpClient(), applicationName, applicationVersion)
        {
        }

        /// <summary>
        /// Public constructor
        /// </summary>
        /// <param name="apiKey">API key</param>
        /// <param name="siteUrl">Site URL</param>
        public WaegisClient(string apiKey, Uri siteUrl)
            : this(apiKey, siteUrl, new HttpClient(), "Waegis Client Library for .NET", _version)
        {
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Initializes a site instance
        /// </summary>
        /// <returns>A Boolean value specifying if the site instance if valid</returns>
        public bool InitializeSite()
        {
            string postData = WaegisSerializer.SerializeSiteInstance(this._siteUrl.ToString());

            string serviceOutput = this._httpClient.PutRequest(this._initializeUrl, this._userAgent, this._timeout, postData);

            ServiceResult result = WaegisDeserializer.GetResultFromData(serviceOutput);

            if (result == null)
                throw new ResponseException("Waegis returned an empty response");

            if (result.Status == ResultStatus.Success)
                return string.Equals("The site is valid.", result.Message, StringComparison.InvariantCultureIgnoreCase);
            else
                throw new ApplicationException(result.Message);
        }

        /// <summary>
        /// Checks a content for being spam or ham
        /// </summary>
        /// <param name="content">The content</param>
        /// <returns>A Boolean value specifying if the content is spam or not</returns>
        public bool CheckContent(Content content)
        {
            string contentData = WaegisSerializer.SerializePost(this._siteUrl.ToString(), content);

            string serviceOutput = this._httpClient.PutRequest(this._checkUrl, this._userAgent, this._timeout, contentData);

            ServiceResult result = WaegisDeserializer.GetResultFromData(serviceOutput);

            if (result == null)
                throw new ResponseException("Waegis returned an empty response");

            if (result.Status == ResultStatus.Failure)
                throw new ResponseException
                    (string.Format(CultureInfo.InvariantCulture, result.Message));

            if (result.Message == "spam")
                return true;
            else
                return false;
        }

        /// <summary>
        /// Reports a ham
        /// </summary>
        /// <param name="content">The content</param>
        /// <returns>A Boolean value indicating if the operation was successful</returns>
        public bool ReportHam(Content content)
        {
            string contentData = WaegisSerializer.SerializePost(this._siteUrl.ToString(), content);

            string serviceOutput = this._httpClient.PutRequest(this._reportHamUrl, this._userAgent, this._timeout, contentData);

            ServiceResult result = WaegisDeserializer.GetResultFromData(serviceOutput);

            if (result == null)
                throw new ResponseException("Waegis returned an empty response");

            if (result.Status == ResultStatus.Failure)
                return false;

            return true;
        }

        /// <summary>
        /// Reports a spam
        /// </summary>
        /// <param name="content">The content</param>
        /// <returns>A Boolean value indicating if the operation was successful</returns>
        public bool ReportSpam(Content content)
        {
            string contentData = WaegisSerializer.SerializePost(this._siteUrl.ToString(), content);

            string serviceOutput = this._httpClient.PutRequest(this._reportSpamUrl, this._userAgent, this._timeout, contentData);

            ServiceResult result = WaegisDeserializer.GetResultFromData(serviceOutput);

            if (result == null)
                throw new ResponseException("Waegis returned an empty response");

            if (result.Status == ResultStatus.Failure)
                return false;

            return true;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Sets the API urls
        /// </summary>
        void SetServiceUrls()
        {
            this._initializeUrl = new Uri(String.Format(CultureInfo.InvariantCulture, initializeUrlFormat, this._apiKey));
            this._checkUrl = new Uri(String.Format(CultureInfo.InvariantCulture, checkUrlFormat, this._apiKey));
            this._reportHamUrl = new Uri(String.Format(CultureInfo.InvariantCulture, reportHamUrlFormat, this._apiKey));
            this._reportSpamUrl = new Uri(String.Format(CultureInfo.InvariantCulture, reportSpamUrlFormat, this._apiKey));
        }

        /// <summary>
        /// Builds the user agent string
        /// </summary>
        /// <param name="applicationName">The name of the application</param>
        /// <param name="applicationVersion">The version of the application</param>
        /// <returns>User agent</returns>
        private string BuildUserAgent(string applicationName, string applicationVersion)
        {
            return string.Format(CultureInfo.InvariantCulture,
                "Waegis Client Library for .NET {0} | {1} {2} | Waegis 1.0", _version, applicationName, applicationVersion);
        }

        #endregion
    }
}
